#include "yosumin.h"
#include "controls.h"
#include "display.h"
#include "utils.h"

#include <stdlib.h>
#include <time.h>

Yosumin::Yosumin()
{
    reset();
}

void Yosumin::reset(void)
{
    level = firstLevelOffset;
    score = 0;
    srand(::time(NULL));
    time = (initialPlayTime - levelBonusPlayTime) * FRAMES_PER_SECOND;
    nextLevel();
}

bool Yosumin::goToNextLevel(void)
{
    int i;

    for ( i = 0 ; i < level / 4 ; i++ )
	if ( eat[i] != 0 )
	    return false;

    return true;
}

void Yosumin::nextLevel()
{
    int i, j;
    int colors;

    level++;

    colors = getColors();

    for ( i = 0 ; i < height ; i++ )
	for ( j = 0 ; j < width ; j++ )
	    grid[i][j] = rand() % colors;

    // Init eat counters
    if ( level < maxColors * 4 )
	for ( i = 0 ; i < colors ; i++ )
	    eat[i] = (level % 4 + 2) * 5;
    else
	for ( i = 0 ; i < colors ; i++ )
	    eat[i] = (level - maxColors * 4) * 5 + 5;

    time += levelBonusPlayTime * FRAMES_PER_SECOND;
}

bool Yosumin::guess(int x1, int y1, int x2, int y2)
{
    this->x1 = x1;
    this->x2 = x2;
    this->y1 = y1;
    this->y2 = y2;

    return grid[y1][x1] == grid[y2][x2]
	&& grid[y1][x2] == grid[y1][x1]
	&& grid[y2][x1] == grid[y2][x2]
	&& x1 != x2 && y1 != y2;
}

void Yosumin::update(void)
{
    int i, j, size;
    int colors = getColors();

    order(x1, y1, x2, y2);

    size = (x2 - x1 + 1) * (y2 - y1 + 1);

    // Update eat table
    eat[grid[y1][x1]] -= size;
    if ( eat[grid[y1][x1]] < 0 )
	eat[grid[y1][x1]] = 0;

    score += (size * time)/10;
    if ( score >= 100000000 ) // Avoid score display overflow
	score = 99999999;

    time += clearBonusPlayTime * FRAMES_PER_SECOND;

    // Make new random squares
    for ( i = y1 ; i <= y2 ; i++ )
	for ( j = x1 ; j <= x2 ; j++ )
	    grid[i][j] = rand() % colors;
}

void Yosumin::timeDown(void)
{
    time--;
}

bool Yosumin::timeUp(void)
{
    return time == 0;
}

unsigned int Yosumin::getTime(void)
{
    return time / FRAMES_PER_SECOND;
}

int Yosumin::getMazeValue(int x, int y)
{
    return grid[y][x];
}

int Yosumin::getEatValue(int p)
{
    return eat[p];
}

unsigned int Yosumin::getScore(void)
{
    return score;
}

unsigned int Yosumin::getLevel(void)
{
    return level - firstLevelOffset;
}

unsigned int Yosumin::getColors(void)
{
    if ( level / 4 > maxColors )
	return maxColors;
    return level / 4;
}

void Yosumin::play(void)
{
    int x1, y1, x2, y2, gotSad;

    YosuminControls * controls = new YosuminControls;
    Display * display = new Display(this, controls);

    display->touchToStart();

    while ( 1 ) {
	controls->update();

	if ( controls->hasPair() ) {
	    controls->getPair(x1, y1, x2, y2);
	    if ( guess(x1, y1, x2, y2) ) {
		display->clear(x1, y1, x2, y2, true);
		gotSad = getMazeValue(x1, y1);
		update();
		display->appear(x1, y1, x2, y2, gotSad);
	    }
	}

	if ( goToNextLevel() ) {
	    display->clearSelection();
	    display->clear();
	    nextLevel();
	    display->appear();
	}

	timeDown();
	if ( timeUp() ) {
	    display->gameover();
	    reset();
	    display->appear();
	}
	if ( getLevel() == 100 ) {
	    display->congrats();
	    reset();
	    display->appear();
	}

	display->display();
    }
}
